/*****************************************************************************
  Pocket BASIC

  |PRBASICC^[v^

  Choi Kyun

  2009-10

  ver 0.1.0
*****************************************************************************/



/*
 * tiny_basic.c
 *
 * main file for tiny_basic
 *
 * This wizard-generated code is based on code adapted from the
 * stationery files distributed as part of the Palm OS SDK 4.0.
 *
 * Copyright (c) 1999-2000 Palm, Inc. or its subsidiaries.
 * All rights reserved.
 */
 
#include <PalmOS.h>
#include <PalmOSGlue.h>
#include <SonyCLIE.h>
#include <ctype.h>


#define GLOBAL_VALUE_DEFINE

/* MathLib headers */
#include "MathLib.h"

#include "tiny_basic.h"
#include "tiny_basic_Rsc.h"
#include "basic.h"



/*********************************************************************
 * Entry Points
 *********************************************************************/
void print_edit_column( void );
void mainform_scroll( UInt16, Int16 );
void mainform_update_scrollbar( UInt16 );
Boolean run_mode_handler( EventType * );
void run_mode_init( FormPtr );
UInt32 get_event_wait( void );
void set_custom_font( void );
void release_custom_font( void );
Boolean check_hard_key( EventPtr );


/*********************************************************************
 * Global variables
 *********************************************************************/


/*********************************************************************
 * Internal Constants
 *********************************************************************/

/* Define the minimum OS version we support */
#define ourMinVersion    sysMakeROMVersion(3,0,0,sysROMStageDevelopment,0)
#define kPalmOS20Version sysMakeROMVersion(2,0,0,sysROMStageDevelopment,0)



/*********************************************************************
 * Internal Functions
 *********************************************************************/



/*
 * FUNCTION: GetObjectPtr
 *
 * DESCRIPTION:
 *
 * This routine returns a pointer to an object in the current form.
 *
 * PARAMETERS:
 *
 * formId
 *     id of the form to display
 *
 * RETURNED:
 *     address of object as a void pointer
 */

void *
GetObjectPtr( UInt16 objectID )
{
  FormType * frmP;


  frmP = FrmGetActiveForm();
  return FrmGetObjectPtr( frmP, FrmGetObjectIndex( frmP, objectID ));
}



/************************************************
 *	n[hL[̃`FbN
 ***********************************************/
static Boolean
check_hard_key( EventPtr event_p )
{
  Boolean handled = false;


  switch( event_p ->eType )
    {
    case keyDownEvent:
      if( TxtGlueCharIsVirtual( event_p ->data.keyDown.modifiers,
                                event_p ->data.keyDown.chr) && g_run_mode )
        {
          switch( event_p ->data.keyDown.chr )
            {
            case vchrHard1:
            case vchrHard2:
            case vchrHard3:
            case vchrHard4:
            case vchrRockerRight:
            case vchrRockerLeft:
            case vchrRockerCenter:
            case vchrPageDown:
            case vchrPageUp:
              handled = true;
              break;
            }
        }
    }

  return handled;
}



// Cxg[ṽEFCg
static UInt32
get_event_wait( void )
{
  return g_run_mode ? 0: evtWaitForever;
}



/*
 * Rg[@vOGA
 */
static void
init_control( void )
{
  ControlPtr control_p;
  static char s[][3] = { "P0", "P1", "P2", "P3", "P4", "P5", "P6", "P7", "P8", "P9" };


  control_p = GetObjectPtr( triger );
  CtlSetLabel( control_p, &s[ g_current_pa ][0] );
}



// |PRtHg
static void
set_custom_font( void )
{
  g_font.h = DmGetResource( fontRscType, pocfont );
  g_font.p = (FontType *) MemHandleLock( g_font.h );
  FntDefineFont( CUSTOM_FONT, g_font.p );

  g_font.old  = FntSetFont( CUSTOM_FONT );
}


static void
release_custom_font( void )
{
  FntSetFont( g_font.old );

  MemHandleUnlock( g_font.h );
  DmReleaseResource( g_font.h );
}



/*
 * XN[o[
 */
static void
mainform_scroll( UInt16 id, Int16 line )
{
  UInt16 blank;
  FieldPtr field_p;


  field_p = GetObjectPtr( id );
  blank = FldGetNumberOfBlankLines( field_p );

  if( line < 0 )
    FldScrollField( field_p, -line, winUp );
  else if( line > 0 )
    FldScrollField( field_p, line, winDown );

  if( blank )
    mainform_update_scrollbar( id );
}



static void
mainform_update_scrollbar( UInt16 id )
{
  UInt16 pos, text_h, field_h;
  Int16 max;
  FieldPtr field_p;
  ScrollBarPtr  bar_p;


  field_p = GetObjectPtr( id ); 
  bar_p = GetObjectPtr( edit_scrollbar );

  FldGetScrollValues( field_p, &pos, &text_h, &field_h );

  if( text_h > field_h)
    {
      max = (text_h - field_h) + FldGetNumberOfBlankLines( field_p );
    }
  else if( pos )
    max = pos;
  else
    max = 0;

  SclSetScrollBar( bar_p, pos, 0, max, field_h - 1);
}



/*
 * GfB^̕\
 */

static void
print_edit_column( void )
{
  char s[8];
  UInt16 size;
  FieldType *field_p;
  RectangleType r = { { 85,2 }, { 45, 10 } };

  if( FrmGetActiveFormID() != MainForm ) return;

  field_p = GetObjectPtr( MainDescriptionField );
  size = FldGetTextLength( field_p );

  if( size != g_last_field_size )
    {
      StrPrintF( s, "%04d:%02d", g_pa_num_lines[ g_current_pa ], FldGetTextLength( field_p ) );
      WinEraseRectangle ( &r, 0 );
      WinDrawChars( s, StrLen( s ), 85, 2 );
      g_last_field_size = size;
    }
}



/*
 * FUNCTION: MainFormInit
 *
 * DESCRIPTION: This routine initializes the MainForm form.
 *
 * PARAMETERS:
 *
 * frm
 *     pointer to the MainForm form.
 */

static void
MainFormInit( FormType *frmP )
{
  FieldType *field;
  char title[40];
  UInt16 fieldIndex;

  TsmFepModeType fep_mode;
  Boolean caps, numlock, auto_shift;
  UInt16 shift;
  char s[2];


  fieldIndex = FrmGetObjectIndex( frmP, MainDescriptionField );
  field = (FieldType *)FrmGetObjectPtr( frmP, fieldIndex );
  FrmSetFocus( frmP, fieldIndex );

  // tHgݒ
//  field = GetObjectPtr( MainDescriptionField );
  FldSetFont( field, CUSTOM_FONT );



  title[0] = '\0';
  StrCat( title, "10 'READY P" );
  s[0] = '\0';
  StrIToA( s, g_current_pa );
  StrCat( title, s );

  if( ! g_pa_last_number[ g_current_pa ] && ! g_err_number )
    FldInsert( field, title, StrLen(title) );
  else if( ! g_err_number )
    {
      g_pa_curr_number = 0; // ݂̕ҏWsZbg
      backward_edit_line();
    }
  else
    {
      g_pa_curr_number = g_err_number;
      g_err_number = 0;
      disp_edit_line();
    }

  // X^bN
  g_go_sp = g_for_sp = 0;
  // tB[h
  g_last_field_size = FldGetTextAllocatedSize( field );
  // Cvbg
  g_input_state = inp_off;

  // FEP OFF 啶Œ 悯ȂbH
  fep_mode = TsmGetFepMode( NULL );
  if( fep_mode != tsmFepModeOff )
    TsmSetFepMode( NULL, tsmFepModeOff );

  GrfGetState( &caps, &numlock, &shift, &auto_shift );
  if( !caps )
    GrfSetState( true, false, false );

  //ҏW
  print_edit_column();
  // t[GA
  update_free_area();
  // R
  g_last_field_size = -1;
  print_edit_column();

  g_key_code = 0;
  WinSetTextColor( INDEX_BLACK );
}



// RUN[h 
static void
run_mode_init( frmP )
{
  FieldPtr field_p = GetObjectPtr( run_input );


  // t[[g
//  g_frame_tick = SysTicksPerSecond() / FRAME_SEC;

  // ݂tick
  g_last_tick = TimGetTicks();

  // s擪̃|C^
  g_line_top = &g_pa_p[ g_current_pa ][0];

  // basicsʒu
  g_program_pointer = &g_pa_p[ g_current_pa ][3];

  // J[\
  g_cursor.x = g_cursor.y = 0;

  // X^bN
  g_go_sp = g_for_sp = 0;

  // Cvbg
  g_input_skip = 0;

  // tHg
  FldSetFont( field_p, CUSTOM_FONT );
}



/*
 * FUNCTION: MainFormDoCommand
 *
 * DESCRIPTION: This routine performs the menu command specified.
 *
 * PARAMETERS:
 *
 * command
 *     menu item id
 */

static Boolean
MainFormDoCommand( UInt16 command )
{
  Boolean handled = false;
  UInt16 length;
  MemHandle h;

  switch( command )
    {
    case sysEditMenuPasteCmd: // y[Xĝ݃nh
      {
        FieldPtr field_p = GetObjectPtr( MainDescriptionField );

        h = ClipboardGetItem( clipboardText, &length );
        if( FldGetTextLength( field_p ) <= 80 - length )
          FldPaste( field_p );
        handled = true;
      }
      break;

    case OptionsAbouttiny_basic:
      {
        FormType * frmP;
        
        /* Clear the menu status from the display */
        MenuEraseStatus( 0 );

        /* Display the About Box. */
        frmP = FrmInitForm( AboutForm );
        FrmDoDialog( frmP );
        FrmDeleteForm( frmP );

        handled = true;
        break;
      }
      /*
    case OptionsPreferences:
      {
        handled = true;
        break;
      }

    case OptionsFont:
      {
        FieldType * field = (FieldType*) GetObjectPtr( MainDescriptionField );
        FontID newFontID = FontSelect( FldGetFont( field ) );
        FldSetFont( field, newFontID );
        handled = true;
        break;
      }
*/
    }

  return handled;
}



/*
 * FUNCTION: MainFormHandleEvent
 *
 * DESCRIPTION:
 *
 * This routine is the event handler for the "MainForm" of this 
 * application.
 *
 * PARAMETERS:
 *
 * eventP
 *     a pointer to an EventType structure
 *
 * RETURNED:
 *     true if the event was handled and should not be passed to
 *     FrmHandleEvent
 */

static Boolean
MainFormHandleEvent( EventType * eventP )
{
  Boolean handled = false;
  FormType * frmP;
  FieldPtr field_p;
  UInt16 index;
  char word[8];
  int val;


//  g_key_code = 0;

  switch( eventP -> eType )
    {
    case menuEvent:
      handled =  MainFormDoCommand( eventP -> data.menu.itemID );
      break;

    case frmOpenEvent:
      frmP = FrmGetActiveForm();
      FrmDrawForm( frmP );
      MainFormInit( frmP );

      init_control(); // vOGA\
      handled = true;
      break;

    case fldChangedEvent: // XN[o[\XV
      mainform_update_scrollbar( eventP -> data.fldChanged.fieldID );
      handled = true;
      break;

    case sclRepeatEvent: // XN[o[XV
      mainform_scroll( MainDescriptionField, eventP -> data.sclRepeat.newValue - eventP -> data.sclRepeat.value );
      break;

    case keyDownEvent:
      field_p = GetObjectPtr( MainDescriptionField );
      if( eventP -> data.keyDown.chr >= 0x20 && eventP -> data.keyDown.chr <= 0x7f )
        {
          if( FldGetTextLength( field_p ) <= 79 )
            handled = FldHandleEvent( field_p, eventP );
          else
            handled = true;
        }
      else if( TxtGlueCharIsVirtual( eventP -> data.keyDown.modifiers, eventP -> data.keyDown.chr ) )
        {
          switch( eventP -> data.keyDown.chr )
            {
            case vchrJogUp:
            case vchrPageUp:
              backward_edit_line();
              handled = true;
              break;

            case vchrJogDown:
            case vchrPageDown:
              forward_edit_line();
              handled = true;
              break;
            }
        }
      break;

    case popSelectEvent: // vOGA؂ւ
      frmP = FrmGetActiveForm();
      field_p = GetObjectPtr( MainDescriptionField );
      g_current_pa = eventP -> data.popSelect.selection;
      FldFreeMemory( field_p );
      FldDrawField( field_p );
      MainFormInit( frmP );
      break;

    case ctlSelectEvent: // e{^
      field_p = GetObjectPtr( MainDescriptionField );
      frmP = FrmGetActiveForm();
      index = FrmGetObjectIndex( frmP, MainDescriptionField ); //xԂ
      FrmSetFocus( frmP, index );

      word[0] = '\0';
      switch( eventP->data.ctlSelect.controlID )
        {
        case btn_run:
          if( g_pa_last_number[ g_current_pa ] )
            {
              g_run_mode = true;
              FrmGotoForm( run_form );
            }
          break;

        case btn_new:
          if( ! FrmAlert( alert_new ) )
            {
              new_list();
              handled = true;
            }
          break;

        case btn_enter:
          conv_ascii_to_basic();
          handled = true;
          break;

        case btn_print:
          StrCat( word, "PRINT " );
          handled = true;
          break;

        case btn_input:
          StrCat( word, "INPUT " );
          handled = true;
          break;

        case btn_if:
          StrCat( word, "IF " );
          handled = true;
          break;

        case btn_then:
          StrCat( word, "THEN " );
          handled = true;
          break;

        case btn_gosub:
          StrCat( word, "GOSUB " );
          handled = true;
          break;

        case btn_return:
          StrCat( word, "RETURN " );
          handled = true;
          break;

        case btn_for:
          StrCat( word, "FOR " );
          handled = true;
          break;

        case btn_to:
          StrCat( word, "TO " );
          handled = true;
          break;

        case btn_goto:
          StrCat( word, "GOTO " );
          handled = true;
          break;

        case btn_vac:
          StrCat( word, "VAC " );
          handled = true;
          break;

        case btn_next:
          StrCat( word, "NEXT " );
          handled = true;
          break;

        case btn_step:
          StrCat( word, "STEP " );
          handled = true;
          break;

        case btn_locate:
          StrCat( word, "CSR " );
          handled = true;
          break;

        case btn_dq:
          StrCat( word, "\"" );
          handled = true;
          break;

        case btn_q:
          StrCat( word, "?" );
          handled = true;
          break;

        case btn_at:
          StrCat( word, "@" );
          handled = true;
          break;

        case btn_dl:
          StrCat( word, "$" );
          handled = true;
          break;

        case btn_cl:
          StrCat( word, ":" );
          handled = true;
          break;

        case btn_scl:
          StrCat( word, ";" );
          handled = true;
          break;

        case btn_cma:
          StrCat( word, "," );
          handled = true;
          break;

        case btn_dot:
          StrCat( word, "." );
          handled = true;
          break;

        case btn_ptn_b:
          StrCat( word, "(" );
          handled = true;
          break;

        case btn_ptn_e:
          StrCat( word, ")" );
          handled = true;
          break;

        case btn_br_b:
          StrCat( word, "<" );
          handled = true;
          break;

        case btn_br_e:
          StrCat( word, ">" );
          handled = true;
          break;

        case btn_eq:
          StrCat( word, "=" );
          handled = true;
          break;

        case btn_pl:
          StrCat( word, "+" );
          handled = true;
          break;

        case btn_mi:
          StrCat( word, "-" );
          handled = true;
          break;

        case btn_mul:
          StrCat( word, "*" );
          handled = true;
          break;

        case btn_div:
          StrCat( word, "/" );
          handled = true;
          break;

        case btn_par:
          StrCat( word, "%" );
          handled = true;
          break;

        case btn_lte:
          StrCat( word, "\xb6" );
          handled = true;
          break;

        case btn_gte:
          StrCat( word, "\xb7" );
          handled = true;
          break;

        case btn_spc:
          StrCat( word, " " );
          handled = true;
          break;

        case btn_cls:
          StrCat( word, "CLS " );
          handled = true;
          break;

        case btn_end:
          StrCat( word, "END " );
          handled = true;
          break;

        case btn_key:
          StrCat( word, "KEY " );
          handled = true;
          break;

        case btn_0:
          StrCat( word, "0" );
          handled = true;
          break;

        case btn_1:
          StrCat( word, "1" );
          handled = true;
          break;

        case btn_2:
          StrCat( word, "2" );
          handled = true;
          break;

        case btn_3:
          StrCat( word, "3" );
          handled = true;
          break;

        case btn_4:
          StrCat( word, "4" );
          handled = true;
          break;

        case btn_5:
          StrCat( word, "5" );
          handled = true;
          break;

        case btn_6:
          StrCat( word, "6" );
          handled = true;
          break;

        case btn_7:
          StrCat( word, "7" );
          handled = true;
          break;

        case btn_8:
          StrCat( word, "8" );
          handled = true;
          break;

        case btn_9:
          StrCat( word, "9" );
          handled = true;
          break;

        case btn_rem:
          StrCat( word, "'" );
          handled = true;
          break;

        case btn_ul:
          StrCat( word, "_" );
          handled = true;
          break;

        case btn_sh:
          StrCat( word, "#" );
          handled = true;
          break;

        case btn_kptn_b:
          StrCat( word, "[" );
          handled = true;
          break;

        case btn_kptn_e:
          StrCat( word, "]" );
          handled = true;
          break;

        case btn_bkr:
          StrCat( word, "!" );
          handled = true;
          break;

        case btn_not:
          StrCat( word, "\xb5" );
          handled = true;
          break;

        case btn_pic1:
          StrCat( word, "\xa1" );
          handled = true;
          break;

        case btn_pic2:
          StrCat( word, "\xa2" );
          handled = true;
          break;

        case btn_pic3:
          StrCat( word, "\xa3" );
          handled = true;
          break;

        case btn_pic4:
          StrCat( word, "\xa4" );
          handled = true;
          break;

        case btn_pic5:
          StrCat( word, "\xa5" );
          handled = true;
          break;

        case btn_pic6:
          StrCat( word, "\xa6" );
          handled = true;
          break;

        case btn_pic7:
          StrCat( word, "\xa7" );
          handled = true;
          break;

        case btn_pic8:
          StrCat( word, "\xa8" );
          handled = true;
          break;

        case btn_pic9:
          StrCat( word, "\xa9" );
          handled = true;
          break;

        case btn_pic10:
          StrCat( word, "\xaa" );
          handled = true;
          break;

        case btn_pic11:
          StrCat( word, "\xab" );
          handled = true;
          break;

        case btn_pic12:
          StrCat( word, "\xac" );
          handled = true;
          break;

        case btn_pic13:
          StrCat( word, "\xad" );
          handled = true;
          break;

        case btn_pic14:
          StrCat( word, "\xae" );
          handled = true;
          break;

        case btn_pic15:
          StrCat( word, "\xaf" );
          handled = true;
          break;

        case btn_pic16:
          StrCat( word, "\xb0" );
          handled = true;
          break;

        case btn_pic17:
          StrCat( word, "\xb1" );
          handled = true;
          break;

        case btn_pic18:
          StrCat( word, "\xb2" );
          handled = true;
          break;

        case btn_pic19:
          StrCat( word, "\xb3" );
          handled = true;
          break;

        case btn_pic20:
          StrCat( word, "\xb4" );
          handled = true;
          break;

        case btn_sin:
          StrCat( word, "SIN" );
          handled = true;
          break;

        case btn_cos:
          StrCat( word, "COS" );
          handled = true;
          break;

        case btn_tan:
          StrCat( word, "TAN" );
          handled = true;
          break;

        case btn_log:
          StrCat( word, "LOG" );
          handled = true;
          break;

        case btn_exp:
          StrCat( word, "EXP" );
          handled = true;
          break;

        case btn_sqr:
          StrCat( word, "SQR" );
          handled = true;
          break;

        case btn_abs:
          StrCat( word, "ABS" );
          handled = true;
          break;

        case btn_int:
          StrCat( word, "INT" );
          handled = true;
          break;

        case btn_ran:
          StrCat( word, "RAN#" );
          handled = true;
          break;

        case btn_frac:
          StrCat( word, "FRAC" );
          handled = true;
          break;

        case btn_10:
          g_pa_curr_number = 0; // ҏWsZbg
          val = g_pa_last_number[ g_current_pa ] + 10;
          StrIToA( word, val );
          StrCat( word, " " );
          FldFreeMemory( field_p );
          FldDrawField( field_p );
          handled = true;
          break;

        case btn_100:
          g_pa_curr_number = 0; // ҏWsZbg
          val = g_pa_last_number [ g_current_pa ] + 100;
          StrIToA( word, val );
          StrCat( word, " " );
          FldFreeMemory( field_p );
          FldDrawField( field_p );
          handled = true;
          break;

        case btn_scroll_up:
          backward_edit_line();
          handled = true;
          break;

        case btn_scroll_down:
          forward_edit_line();
          handled = true;
          break;

        case btn_del_line:
          if( g_pa_curr_number )
            {
              delete_line( g_pa_curr_number );
              FldFreeMemory( field_p );
              FldDrawField( field_p );
              update_free_area();
              handled = true;
            }
          break;
        }
      if( FldGetTextLength( field_p ) <= (80 - StrLen(word)) )
        FldInsert( field_p, word, StrLen(word) );
    }

  return handled;
}



// RUN[h nh
static Boolean
run_mode_handler( EventType * eventP )
{
  Boolean handled = false;
  FormType * frmP;
  FieldPtr field_p;
  char *src_p;


  switch( eventP -> eType )
    {
    case nilEvent:
      execute(); // basic s
      handled = true;
      break;

    case menuEvent:
      break;

    case frmOpenEvent:
      frmP = FrmGetActiveForm();
      FrmDrawForm( frmP );
      run_mode_init( frmP );

      handled = true;
      break;

    case keyDownEvent:
      g_key_code = eventP -> data.keyDown.chr;
//      handled = true;
      break;

    case penDownEvent:
      break;

    case ctlSelectEvent: // e{^
      switch( eventP->data.ctlSelect.controlID )
        {
        case btn_stop:
          g_run_mode = false;
          FrmGotoForm( MainForm );

          handled = true;
          break;

        case btn_input:
          g_input_state = inp_end;
          field_p = GetObjectPtr( run_input );
          src_p = FldGetTextPtr( field_p );
          if( src_p )
            StrCopy( g_input_buf, src_p );
          flash_field( field_p );
          handled = true;
          break;
        }
    }

  return handled;
}



/*
 * FUNCTION: AppHandleEvent
 *
 * DESCRIPTION: 
 *
 * This routine loads form resources and set the event handler for
 * the form loaded.
 *
 * PARAMETERS:
 *
 * event
 *     a pointer to an EventType structure
 *
 * RETURNED:
 *     true if the event was handled and should not be passed
 *     to a higher level handler.
 */

static Boolean
AppHandleEvent( EventType * eventP )
{
  UInt16 formId;
  FormType * frmP;

  if( eventP -> eType == frmLoadEvent )
    {
      /* Load the form resource. */
      formId = eventP -> data.frmLoad.formID;
      frmP = FrmInitForm( formId );
      FrmSetActiveForm( frmP );

      switch( formId )
        {
        case MainForm:
          FrmSetEventHandler( frmP, MainFormHandleEvent );
          break;

        case run_form:
          FrmSetEventHandler( frmP, run_mode_handler  );
          break;
        }
      return true;
    }

  return false;
}



/*
 * FUNCTION: AppEventLoop
 *
 * DESCRIPTION: This routine is the event loop for the application.
 */

static void
AppEventLoop( void )
{
  UInt16 error;
  EventType event;

  do
    {
      EvtGetEvent( &event, get_event_wait() );

      print_edit_column(); // c蕶

      // n[hL[̓XLbv
      if( !check_hard_key( &event ) )
        if( SysHandleEvent( &event ) ) continue;

      if( !MenuHandleEvent( 0, &event, &error ) )
        if( !AppHandleEvent( &event ) )
          FrmDispatchEvent( &event );
    }
  while( event.eType != appStopEvent );
}



/*
 * FUNCTION: AppStart
 *
 * DESCRIPTION:  Get the current application's preferences.
 *
 * RETURNED:
 *     errNone - if nothing went wrong
 */

static Err
AppStart( void )
{
  int ver;
  UInt16 size;
  PocketBasicPreferenceType pref;
  Err err = errNone;


  /* load the MathLib library, save reference in MathLibRef */
  {
    MathLibRef = sysInvalidRefNum;

    if( errNone != SysLibFind( MathLibName, &MathLibRef ) )
      {
        SysLibLoad( LibType, MathLibCreator, &MathLibRef );
      }

    if( MathLibRef != sysInvalidRefNum )
      {
        if( mlErrNone != MathLibOpen( MathLibRef, MathLibVersion ) )
          {
            /* if error loading library, reset reference */
            MathLibRef = sysInvalidRefNum;
          }
      }
  }
  // Cu
  if( MathLibRef == sysInvalidRefNum )
    {
      FrmAlert( lib_error );
      return -1;
    }

  // vOGA
  init_program_area();

  // [Nm
  // vOGA
  g_pa_p = (char (*)[MAX_PROGRAM_SIZE])MemPtrNew( sizeof(char) * MAX_PROGRAM_AREA_NUM * MAX_PROGRAM_SIZE );
  ErrFatalDisplayIf( ! g_pa_p, "Error allocating Buffers" );
  MemSet( g_pa_p, sizeof(char) * MAX_PROGRAM_AREA_NUM * MAX_PROGRAM_SIZE, 0 );

  // vOGAf[^x[X
  err = open_program_area_db();
  if( err )
    {
      FrmAlert( PaDBError );
      return err;
    }

  // vt@X
  size = sizeof( PocketBasicPreferenceType );
  ver = PrefGetAppPreferences( appFileCreator, appPrefID, &pref, &size, true );
  if( ver == noPreferenceFound
      || ver != appPrefVersionNum
      || size != sizeof( PocketBasicPreferenceType ) )
    {
      // vOGA
      init_program_area();
      // f[^x[X
      close_program_area_db();
      delete_program_area_db();
      err = open_program_area_db();
      if( err )
        {
          FrmAlert( PaDBError );
          return err;
        }
    }
  else
    {
      // vt@X̃[h
      g_current_pa = pref.current_pa;
      MemMove( g_pa_free_size, pref.pa_free_size, sizeof(g_pa_free_size) );
      MemMove( g_pa_last_number, pref.pa_last_number, sizeof(g_pa_last_number) );
      MemMove( g_pa_last_ptr, pref.pa_last_ptr, sizeof(g_pa_last_ptr) );
      MemMove( g_pa_num_lines, pref.pa_num_lines, sizeof(g_pa_num_lines) );
    }

  // G[
  g_err_number = 0;
  // ϐ̈揉
  init_variable();

  // tHgZbg
  set_custom_font();

  WinPushDrawState();

  return errNone;
}



/*
 * FUNCTION: AppStop
 *
 * DESCRIPTION: Save the current state of the application.
 */

static void
AppStop( void )
{
  PocketBasicPreferenceType pref;


  /* unload the MathLib library */
  if( MathLibRef != sysInvalidRefNum )
    {
      UInt16 useCount;

      MathLibClose( MathLibRef, &useCount );
      if( useCount == 0 )
        {
          SysLibRemove( MathLibRef );
        }

      MathLibRef = sysInvalidRefNum;
    }

  // vt@X̃Z[u
  pref.current_pa = g_current_pa;
  MemMove( pref.pa_free_size, g_pa_free_size, sizeof(g_pa_free_size) );
  MemMove( pref.pa_last_number, g_pa_last_number, sizeof(g_pa_last_number) );
  MemMove( pref.pa_last_ptr, g_pa_last_ptr, sizeof(g_pa_last_ptr) );
  MemMove( pref.pa_num_lines, g_pa_num_lines, sizeof(g_pa_num_lines) );

  PrefSetAppPreferences( appFileCreator, appPrefID, appPrefVersionNum,
                         &pref, sizeof( PocketBasicPreferenceType ), true );

  // f[^x[X
  close_program_area_db();
  // [N
  MemPtrFree( g_pa_p );

  // tHg߂
  release_custom_font();

  /* Close all the open forms. */
  FrmCloseAllForms();

  WinPopDrawState();
}



/*
 * FUNCTION: RomVersionCompatible
 *
 * DESCRIPTION: 
 *
 * This routine checks that a ROM version is meet your minimum 
 * requirement.
 *
 * PARAMETERS:
 *
 * requiredVersion
 *     minimum rom version required
 *     (see sysFtrNumROMVersion in SystemMgr.h for format)
 *
 * launchFlags
 *     flags that indicate if the application UI is initialized
 *     These flags are one of the parameters to your app's PilotMain
 *
 * RETURNED:
 *     error code or zero if ROM version is compatible
 */

static Err
RomVersionCompatible( UInt32 requiredVersion, UInt16 launchFlags )
{
  UInt32 romVersion;

  /* See if we're on in minimum required version of the ROM or later. */
  FtrGet( sysFtrCreator, sysFtrNumROMVersion, &romVersion );
  if( romVersion < requiredVersion )
    {
      if( ( launchFlags & (sysAppLaunchFlagNewGlobals | sysAppLaunchFlagUIApp) ) ==
          (sysAppLaunchFlagNewGlobals | sysAppLaunchFlagUIApp) )
        {
          FrmAlert( RomIncompatibleAlert );

          /* Palm OS versions before 2.0 will continuously relaunch this
           * app unless we switch to another safe one. */
          if( romVersion < kPalmOS20Version )
            {
              AppLaunchWithCommand( sysFileCDefaultApp, sysAppLaunchCmdNormalLaunch, NULL );
            }
        }

      return sysErrRomIncompatible;
    }

  return errNone;
}



/*
 * FUNCTION: PilotMain
 *
 * DESCRIPTION: This is the main entry point for the application.
 * 
 * PARAMETERS:
 *
 * cmd
 *     word value specifying the launch code. 
 *
 * cmdPB
 *     pointer to a structure that is associated with the launch code
 *
 * launchFlags
 *     word value providing extra information about the launch.
 *
 * RETURNED:
 *     Result of launch, errNone if all went OK
 */

UInt32
PilotMain( UInt16 cmd, MemPtr cmdPBP, UInt16 launchFlags )
{
  Err error;

  error = RomVersionCompatible( ourMinVersion, launchFlags );
  if( error ) return( error );

  switch( cmd )
    {
    case sysAppLaunchCmdNormalLaunch:
      error = AppStart();
      if( error ) 
        return error;

      /* 
       * start application by opening the main form
       * and then entering the main event loop 
       */
      FrmGotoForm( MainForm );
      AppEventLoop();

      AppStop();
      break;
    }

  return errNone;
}



